#ifndef	_SERV_H
#define	_SERV_H
/*
 * Copyright (c) 1991 David I. Bell
 * Permission is granted to use, distribute, or modify this source,
 * provided that this copyright notice remains intact.
 *
 * Private definitions for the graphics server.
 * These definitions are not to be used by clients.
 */
#include "nano-X.h"

/*
 * Drawing types.
 */
typedef	int	GR_DRAW_TYPE;

#define	GR_DRAW_TYPE_NONE	((GR_DRAW_TYPE) 0)	/* none or error */
#define	GR_DRAW_TYPE_WINDOW	((GR_DRAW_TYPE) 1)	/* windows */
#define	GR_DRAW_TYPE_PIXMAP	((GR_DRAW_TYPE) 2)	/* pixmaps */

/*
 * List of elements for events.
 */
typedef	struct gr_event_list GR_EVENT_LIST;
struct gr_event_list {
	GR_EVENT_LIST	*next;		/* next element in list */
	GR_EVENT	event;		/* event */
};

/*
 * Data structure to keep track of state of clients.
 */
typedef struct gr_client GR_CLIENT;
struct gr_client {
	int		id;		/* client id and socket descriptor */
	GR_EVENT_LIST	*eventhead;	/* head of event chain (or NULL) */
	GR_EVENT_LIST	*eventtail;	/* tail of event chain (or NULL) */
	GR_EVENT_ERROR	errorevent;	/* waiting error event to deliver */
	GR_CLIENT	*next;		/* the next client in the list */
	GR_CLIENT	*prev;		/* the previous client in the list */
	int		waiting_for_event; /* used to implement GrGetNextEvent*/
};

/*
 * Structure to remember clients associated with events.
 */
typedef	struct gr_event_client	GR_EVENT_CLIENT;
struct gr_event_client	{
	GR_EVENT_CLIENT	*next;		/* next interested client or NULL */
	GR_EVENT_MASK	eventmask;	/* events client wants to see */
	GR_CLIENT	*client;	/* client who is interested */
};

/*
 * Structure to remember graphics contexts.
 */
typedef	struct gr_gc	GR_GC;
struct gr_gc {
	GR_GC_ID	id;		/* graphics context id */
	GR_MODE		mode;		/* drawing mode */
	GR_FONT		font;		/* current font number */
	GR_COLOR	foreground;	/* foreground color */
	GR_COLOR	background;	/* background color */
	GR_BOOL		usebackground;	/* actually display the background */
	GR_BOOL		changed;	/* graphics context has been changed */
	GR_GC		*next;		/* next graphics context */
	int		usecount;	/* usecount */
};

/*
 * Structure to remember cursor definitions.
 */
typedef	struct {
	int		usecount;	/* use counter */
	SWCURSOR	cursor;
} GR_CURSOR;


/*
 * Window structure
 */
typedef struct gr_window GR_WINDOW;
struct gr_window {
	GR_WINDOW_ID	id;		/* window id */
	GR_WINDOW	*parent;	/* parent window */
	GR_WINDOW	*children;	/* first child window */
	GR_WINDOW	*siblings;	/* next sibling window */
	GR_WINDOW	*next;		/* next window in complete list */
	GR_COORD	x;		/* absolute x position */
	GR_COORD	y;		/* absolute y position */
	GR_SIZE		width;		/* width */
	GR_SIZE		height;		/* height */
	GR_SIZE		bordersize;	/* size of border */
	GR_COLOR	bordercolor;	/* color of border */
	GR_COLOR	background;	/* background color */
	GR_EVENT_MASK	nopropmask;	/* events not to be propagated */
	GR_EVENT_CLIENT	*eventclients;	/* clients interested in events */
	GR_CURSOR	*cursor;	/* cursor for this window */
	GR_BOOL		mapped;		/* TRUE if explicitly mapped */
	GR_COUNT	unmapcount;	/* count of reasons not really mapped */
	GR_BOOL		output;		/* TRUE if window can do output */
};


/*
 * Pixmap structure (not yet implemented).
 * Pixmaps of 2 colors use bitmaps, otherwise they use color values.
 */
typedef struct gr_pixmap	GR_PIXMAP;
struct gr_pixmap {
	GR_PIXMAP_ID	id;		/* pixmap id */
	GR_PIXMAP	*next;		/* next pixmap in list */
	GR_SIZE		width;		/* width of pixmap */
	GR_SIZE		height;		/* height of pixmap */
	GR_COLOR	maxcolor;	/* maximum color used in pixmap */
	GR_COLOR	*colors;	/* table of color values */
	GR_BITMAP	*bitmap;	/* OR: table of bitmap values */
};


/*
 * Macros to obtain the client number from a resource id, and to
 * produce the first resource id to be used for a client number.
 * Client numbers must not be zero.  This allows for 255 clients.
#define	GR_ID_CLIENT(n)	(((GR_ID) (n)) >> 24)
#define	GR_ID_BASE(n)	(((GR_ID) (n)) << 24)
 */


/*
 * Graphics server routines.
 */
int		GsInitialize(void);
void		GsSelect(void);
void		GsTerminate(void);
void		GsRedrawScreen(void);
void		GsError(GR_ERROR code, GR_ID id);
GR_BOOL		GsCheckMouseEvent(void);
GR_BOOL		GsCheckKeyboardEvent(void);
int		GsReadKeyboard(GR_CHAR *buf, MODIFIER *modifiers);
int		GsOpenKeyboard(void);
void		GsGetButtonInfo(BUTTON *buttons);
void		GsGetModifierInfo(MODIFIER *modifiers);
void		GsCloseKeyboard(void);
void		GsExposeArea(GR_WINDOW *wp, GR_COORD rootx, GR_COORD rooty,
			GR_SIZE width, GR_SIZE height);
void		GsCheckCursor(void);
void		GsWpSetFocus(GR_WINDOW *wp);
void		GsWpClearWindow(GR_WINDOW *wp, GR_COORD x, GR_COORD y,
			GR_SIZE width, GR_SIZE height, GR_BOOL exposeflag);
void		GsWpUnmapWindow(GR_WINDOW *wp);
void		GsWpMapWindow(GR_WINDOW *wp);
void		GsWpDestroyWindow(GR_WINDOW *wp);
void		GsSetClipWindow(GR_WINDOW *wp);
GR_COUNT	GsSplitClipRect(CLIPRECT *srcrect, CLIPRECT *destrect,
			GR_COORD minx, GR_COORD miny, GR_COORD maxx,
			GR_COORD maxy);
void		GsHandleMouseStatus(GR_COORD newx, GR_COORD newy,
			BUTTON newbuttons);
void		GsFreePositionEvent(GR_CLIENT *client, GR_WINDOW_ID wid,
			GR_WINDOW_ID subwid);
void		GsDeliverButtonEvent(GR_EVENT_TYPE type, BUTTON buttons,
			BUTTON changebuttons, MODIFIER modifiers);
void		GsDeliverMotionEvent(GR_EVENT_TYPE type, BUTTON buttons,
			MODIFIER modifiers);
void		GsDeliverKeyboardEvent(GR_EVENT_TYPE type, GR_CHAR ch,
			MODIFIER modifiers);
void		GsDeliverExposureEvent(GR_WINDOW *wp, GR_COORD x, GR_COORD y,
			GR_SIZE width, GR_SIZE height);
void		GsDeliverGeneralEvent(GR_WINDOW *wp, GR_EVENT_TYPE type);
void		GsCheckMouseWindow(void);
void		GsCheckFocusWindow(void);
GR_DRAW_TYPE	GsPrepareDrawing(GR_DRAW_ID id, GR_GC_ID gcid,
			GR_WINDOW **retwp, GR_PIXMAP **retpixptr);
GR_BOOL		GsCheckOverlap(GR_WINDOW *topwp, GR_WINDOW *botwp);
GR_EVENT	*GsAllocEvent(GR_CLIENT *client);
GR_WINDOW	*GsFindWindow(GR_WINDOW_ID id);
GR_GC		*GsFindGC(GR_GC_ID gcid);
GR_WINDOW	*GsPrepareWindow(GR_WINDOW_ID wid);
GR_WINDOW	*GsFindVisibleWindow(GR_COORD x, GR_COORD y);
void		GsDrawBorder(GR_WINDOW *wp);
int		GsCurrentVt(void);
void		GsRedrawVt(int t);
int		GsOpenSocket(void);
void		GsCloseSocket(void);
void		GsAcceptClient(void);
void		GsAcceptClientFd(int i);
int		GsPutCh(int fd, unsigned char c);
GR_CLIENT	*GsFindClient(int fd);
void		GsDropClient(int fd);
int		GsRead(int fd, void *buf, int c);
int		GsWrite(int fd, void *buf, int c);
void		GsHandleClient(int fd);

void		GrOpenWrapper(void *);
void 		GrCloseWrapper(void *);
void		GrGetScreenInfoWrapper(void *);
void		GrSetErrorHandlerWrapper(void *);
void		GrNewWindowWrapper(void *);
void		GrNewInputWindowWrapper(void *);
void		GrDestroyWindowWrapper(void *);
void		GrNewGCWrapper(void *);
void		GrGetGCInfoWrapper(void *);
void		GrDestroyGCWrapper(void *);
void		GrMapWindowWrapper(void *);
void		GrUnmapWindowWrapper(void *);
void		GrRaiseWindowWrapper(void *);
void		GrLowerWindowWrapper(void *);
void		GrMoveWindowWrapper(void *);
void		GrResizeWindowWrapper(void *);
void		GrReparentWindowWrapper(void *);
void		GrGetWindowInfoWrapper(void *);
void		GrGetFontInfoWrapper(void *);
void		GrSetFocusWrapper(void *);
void		GrSetBorderColorWrapper(void *);
void		GrClearWindowWrapper(void *);
void		GrSelectEventsWrapper(void *);
void		GrGetNextEventWrapper(void *);
void		GrGetNextEventWrapperFinish(int fd);
void		GrCheckNextEventWrapper(void *);
void		GrPeekEventWrapper(void *);
void		GrLineWrapper(void *);
void		GrPointWrapper(void *);
void		GrRectWrapper(void *);
void		GrFillRectWrapper(void *);
void		GrPolyWrapper(void *);
void		GrFillPolyWrapper(void *);
void		GrEllipseWrapper(void *);
void		GrFillEllipseWrapper(void *);
void		GrSetGCForegroundWrapper(void *);
void		GrSetGCBackgroundWrapper(void *);
void		GrSetGCUseBackgroundWrapper(void *);
void		GrSetGCModeWrapper(void *);
void		GrSetGCFontWrapper(void *);
void		GrGetGCTextSizeWrapper(void *);
void		GrReadArea8Wrapper(void *);
void		GrArea8Wrapper(void *);
void		GrBitmapWrapper(void *);
void		GrTextWrapper(void *);
void		GrSetCursorWrapper(void *);
void		GrMoveCursorWrapper(void *);
void		GrGetSystemPaletteWrapper(void *);
void		GrFindColorWrapper(void *);
void		GrJPEGWrapper(void *);
void		GrBMPWrapper(void *);

/*
 * External data definitions.
 */
extern	char *		curfunc;		/* current function name */
extern	GR_WINDOW_ID	cachewindowid;		/* cached window id */
extern	GR_GC_ID	cachegcid;		/* cached graphics context id */
extern	GR_GC		*cachegcp;		/* cached graphics context */
extern	GR_GC		*listgcp;		/* list of all gc */
extern	GR_GC		*curgcp;		/* current graphics context */
extern	GR_WINDOW	*cachewp;		/* cached window pointer */
extern	GR_WINDOW	*listwp;		/* list of all windows */
extern	GR_WINDOW	*rootwp;		/* root window pointer */
extern	GR_WINDOW	*clipwp;		/* window clipping is set for */
extern	GR_WINDOW	*focuswp;		/* focus window for keyboard */
extern	GR_WINDOW	*mousewp;		/* window mouse is currently in */
extern	GR_WINDOW	*grabbuttonwp;		/* window grabbed by button */
extern	GR_CURSOR	*curcursor;		/* currently enabled cursor */
extern	GR_COORD	cursorx;		/* x position of cursor */
extern	GR_COORD	cursory;		/* y position of cursor */
extern	BUTTON	curbuttons;			/* current state of buttons */
extern	GR_CLIENT	*curclient;		/* current client */
extern	GR_EVENT_LIST	*eventfree;		/* list of free events */
extern	GR_BOOL		focusfixed;		/* TRUE if focus is fixed */
extern	GR_SCREEN_INFO	sinfo;			/* screen information */
extern	GR_FONT_INFO	curfont;		/* current font information */

/*
 * The filename to use for the named socket. If we ever support multiple
 * servers on one machine, the last digit will be that of the FB used for it.
 */
#define GR_NAMED_SOCKET "/tmp/.nano-X"

#if FRAMEBUFFER | BOGL
/* temp framebuffer vt switch stuff at upper level
 * this should be handled at the lower level, just like vgalib does.
 */
void fb_InitVt(void);
int  fb_CurrentVt(void);
int  fb_CheckVtChange(void);
void fb_RedrawVt(int t);
void fb_ExitVt(void);
extern int vterm;
#endif /* FRAMEBUFFER | BOGL*/

#endif /* _SERV_H*/
